#ifndef L2_BUFFMAP
#define L2_BUFFMAP

#include "l2geo.h"
#include "rps.h"
//#include "l2defs.h"
#include "pltable.h"

#define L2FR_BM_ENTRIES 25000 /* the number of fed's readouts */
#define L2FR_N_SECTIONS   18 /*This will go to 20 in the new geometry*/
#define L2FR_N_LAYERS   MAX_PLANES /* == 5000 in Rev0501 */
#define N_LAYPERSECT 4


typedef struct L2FR_DET_Entry L2FR_DET_Entry;

struct L2FR_BM_Entry { 
  char       spl[4];    /* superlayer name */
  int        lay;       /* relative layer */
  int        sec;       /* relative sector */
  int        mod;       /* relative module */
  int        modtype ;  /* module type for Otr */
  int        chan1;     /* 1st channel on FED */
  int        chan2;     /* last channel on FED */
  int        wire1;     /* number of 1st wire on GEDE */
  int        wire2;     /* number of last wire on GEDE */
  int        rrb_id;
  unsigned char chip_pos; /* position of sharc chip  */
  unsigned char vme_id;   /* the crate controller number (e.g. the 40 of hb-vme40) */
  unsigned char vme_addr; /* as fixed on board */
  unsigned char cid;      /*  component id */
  int        copy;     
  int        link; 
  RPS_addr   addr_send;   /* dinamically assigned */
  RPS_addr   addr_range;  /* dinamically assigned */
  RPS_addr   send_order;  /* dinamically assigned */
};

typedef struct L2FR_BM_Entry L2FR_BM_Entry;

struct L2FR_BMSectMap { /* needed ? */
  float xMin[2];
  float xMax[2];
  float yMin[2];
  float yMax[2];
  float zWin[2];
  int   nLayers;
  int   layer[N_LAYPERSECT];
};
typedef struct L2FR_BMSectMap L2FR_BMSectMap;

struct L2FR_BMLayerMap { /* structure needed to connect geo to buffer map */
  char  spl[4];
  int   sec;
  int   lay;
  int   mod;
  int   modtype ;  /* module type for Otr */
  int   cmp;
  float xMin; /* These numbers should indicate the edge of the active area */
  float xMax;
  float yMin;
  float yMax;
  float z;
  float cos; /* ask mike. I need to catch strips x<xmin,ymin<y<ymax */
  float sin;
  float pitch;
  int nEntries; /* in buffermap */
  int first;    /* offset to first entry */
  PLANE* Pl;    /* Pointer to PLANE table */
};
typedef struct L2FR_BMLayerMap L2FR_BMLayerMap;

int init_BuffMapFromBinary(char *fName);
int init_BuffMapFromAscii(char *fName);
int init_BuffMapFromPipe();

int MakeMaps(void);
int MakeSectMap();
int MakeLayerMap();

const L2FR_BM_Entry *BM_getBuffMap(void);
const L2FR_BMSectMap *BM_getSectorMap(void);
const L2FR_BMLayerMap *BM_getLayerMap(void);

void L2fr_BuffMapPrintSectMap(FILE * );
void L2fr_BuffMapPrintLayerMap(FILE * );
void L2fr_BuffMapPrintTable(FILE * );
void L2fr_BuffMapPrintEntry(L2FR_BM_Entry *, FILE *);

typedef enum {L2_XGEO,L2_STEREOGEO, L2_MAGNGEO, L2_VTXGEO} GEOTYPE;

unsigned int LookupID(L2FR_BM_Entry *map);

L2FR_BMLayerMap  **Lay_GetBack( GEOTYPE , int *);

#endif



