/***
 *
 * Mnemonics for specifying detector types. Intended for use with
 * register directives.
 *
 * ... and superlayer names
 *
 ***/

#include <string.h>

#define  _DTTABLE_DOT_C
#include "dttable.h"

typedef struct {
	char *mnem;	/* mnemonic */
	DTTYPE type;	/* as defined in HBGean ...well usually */
} DETTYP;

static DETTYP detTypes[] = {
	{ "SILICON", SILICON},
	{ "OUTER_TRACKER", OUTER_TRACKER},
	{ "MSGC", MSGC},
	{ "PADS", PADS},
	{ "ECAL", ECAL},
	{ "MUON_TRACKER", MUON_TRACKER},
	{ "MUON_PADS", MUON_PADS},
	{ "LEVEL1", LEVEL1},
        { "", ERROR}
};

const char nSI01[4] = {'S', 'I', '0', '1'}; 
const char nSI02[4] = {'S', 'I', '0', '2'}; 
const char nSI03[4] = {'S', 'I', '0', '3'}; 
const char nSI04[4] = {'S', 'I', '0', '4'}; 
const char nSI05[4] = {'S', 'I', '0', '5'}; 
const char nSI06[4] = {'S', 'I', '0', '6'}; 
const char nSI07[4] = {'S', 'I', '0', '7'}; 
const char nSI08[4] = {'S', 'I', '0', '8'}; 
const char nSI09[4] = {'S', 'I', '0', '9'}; 
const char nSI10[4] = {'S', 'I', '1', '0'}; 
const char nMC01[4] = {'M', 'C', '0', '1'}; 
const char nMC02[4] = {'M', 'C', '0', '2'}; 
const char nMC03[4] = {'M', 'C', '0', '3'}; 
const char nMC04[4] = {'M', 'C', '0', '4'}; 
const char nMC05[4] = {'M', 'C', '0', '5'}; 
const char nMC06[4] = {'M', 'C', '0', '6'}; 
const char nMC07[4] = {'M', 'C', '0', '7'}; 
const char nMC08[4] = {'M', 'C', '0', '8'}; 
const char nMC09[4] = {'M', 'C', '0', '9'}; 
const char nMS01[4] = {'M', 'S', '0', '1'}; 
const char nMS03[4] = {'M', 'S', '0', '3'}; 
const char nMS05[4] = {'M', 'S', '0', '5'}; 
const char nMS07[4] = {'M', 'S', '0', '7'}; 
const char nMS09[4] = {'M', 'S', '0', '9'}; 
const char nMS10[4] = {'M', 'S', '1', '0'}; 
const char nMS11[4] = {'M', 'S', '1', '1'}; 
const char nMS12[4] = {'M', 'S', '1', '2'}; 
const char nMS13[4] = {'M', 'S', '1', '3'}; 
const char nMS14[4] = {'M', 'S', '1', '4'}; 
const char nMS15[4] = {'M', 'S', '1', '5'}; 
const char nPTL1[4] = {'P', 'T', 'L', '1'}; 
const char nPTL2[4] = {'P', 'T', 'L', '2'}; 
const char nPTL3[4] = {'P', 'T', 'L', '3'}; 
const char nPC01[4] = {'P', 'C', '0', '1'}; 
const char nPC02[4] = {'P', 'C', '0', '2'}; 
const char nPC03[4] = {'P', 'C', '0', '3'}; 
const char nPC04[4] = {'P', 'C', '0', '4'}; 
const char nTC01[4] = {'T', 'C', '0', '1'}; 
const char nTC02[4] = {'T', 'C', '0', '2'}; 
const char nDETR[4] = {'D', 'E', 'T', 'R'}; 
const char nCALW[4] = {'C', 'A', 'L', 'W'}; 
const char nMUD1[4] = {'M', 'U', 'D', '1'}; 
const char nMUD2[4] = {'M', 'U', 'D', '2'}; 
const char nMUD3[4] = {'M', 'U', 'D', '3'}; 
const char nMUD4[4] = {'M', 'U', 'D', '4'}; 
const char nMXD1[4] = {'M', 'X', 'D', '1'}; 
const char nMXD2[4] = {'M', 'X', 'D', '2'}; 
const char nMXD3[4] = {'M', 'X', 'D', '3'}; 
const char nMXD4[4] = {'M', 'X', 'D', '4'}; 

/***
 *
 * DTLookup
 *
 * Lookup detector type. Return type number or zero if not found.
 *
 ***/

int DTLookup( char *mnemonic)
{
	int i;

	for( i = 0; detTypes[i].mnem != '\0'; i++)
		if( strcmp(detTypes[i].mnem, mnemonic) == 0)
			break;

	return detTypes[i].type;
}

