#include <stdio.h>
#include <stdlib.h>
#include "string.h"
#include "l2geo.h"
#include "grr.h"
#include "sltable.h"
#include "pltable.h"
#include "l2mode.h"
#include "rpm.h"
#include "pub.h"

static int scenario98 = -1; /* -1 means standard hera-b */

static L2GEO_VERSION dummy_version = {0}; /* ...needs attention */

/***
 *
 * GetGeo
 *
 * Return a copy of the geometry record for plane, or NULL if error.
 * If geoRec is NULL on entry, a new record is created otherwise, the
 * requested record is copied into geoRec.
 *
 ***/
 
PLANE *GetGeo(int plane, PLANE *geoRec)
{
	PLANE *aRec;
	char mess[200];

	if ( (aRec = PlLookup(plane)) == NULL)
	  return NULL;

/* Allocate space, if called for */

	if( geoRec == NULL) {
	  geoRec = malloc( sizeof( PLANE ));
	  if( geoRec == NULL) {
	    sprintf(mess,"GetGeo: unable to allocate space for geometry record.\n");
	    grr_error(GRR_SFATAL,mess,1);
	    exit(EXIT_FAILURE);
	  }
	}
	
	*geoRec = *aRec;

	return geoRec;		
}

void Geo_PrintPlane(PLANE *thePlane)
{
	printf("Geometry record for plane %d\n", thePlane->l2npl);
	printf("superlayer number: %d, mother volume: %d\n", thePlane->shp, thePlane->cmp);
	printf("cos, sin of angle between x-axis and measurement direction: %f, %f\n",thePlane->cos,thePlane->sin);
	printf("lower and upper limit in measurement direction: %f, %f\n",thePlane->sxmn, thePlane->sxmx);
	printf("lower and upper limit in orthogonal  direction: %f, %f\n",thePlane->symn, thePlane->symx);
	printf("z position: %f\n", thePlane->zfw[0]);
}

/***
 *
 * Geo_GetBack
 *
 * Return an array of pointers to L2GEO records for
 * planes behind the magnet...inner and outer. Depending
 * on the value of which return GEO_GET_ALL, GEO_GET_X,
 * GEO_GET_STEREO.
 *
 * "Behind the magnet" means a PC, TC chamber or an MS
 * chamber >= MS10
 *
 ***/

PLANE **Geo_GetBack( GEO_WHICH which, int *nPlanes)
{
  int nPl = 0;
  PLANE **plArray, *aPl;
  SUPERLAYER *itsSL;
  char mess[200];
    
  /* first count the number of planes */

  for(aPl = Pl_IterateInit(NULL); aPl != NULL; aPl = Pl_IterateNext()) {

    if( aPl->cmp == MSGC || aPl->cmp == OUTER_TRACKER ) {

       if( (itsSL = SLLookup_bp(aPl->cmp, aPl->l2npl)) == NULL ) {
	 sprintf(mess,"Geo_GetBack: Something's seriously wrong\n");
	 grr_error(GRR_SFATAL,mess,1);
	 exit(EXIT_FAILURE);
       }

       if((itsSL->name[0] == 'P' && itsSL->name[1] == 'C') ||
          (itsSL->name[0] == 'T' && itsSL->name[1] == 'C') ||
          (itsSL->name[0] == 'M' && itsSL->name[1] == 'S' && itsSL->name[2] == '1')) {

        switch (which) {

        case GEO_GET_ALL:
	  nPl++;
	  break;

        case GEO_GET_X:
	  if ( aPl->sin > -.00001 && aPl->sin < .00001)
	    nPl++;
	  break;

        case GEO_GET_STEREO:
	  if ( aPl->sin < -.00001 || aPl->sin > .00001)
	    nPl++;
	  break;
        }
      }
    }
  }

  *nPlanes = nPl;

  /* allocate memory */

    if( (plArray = malloc( nPl * sizeof(PLANE *) )) == NULL) {
      sprintf(mess,"Geo_GetXBack: unable to allocate memory\n");
      grr_error(GRR_SFATAL,mess,1);
    }
  
  /* fill 'er up */

  nPl = 0;     

  for(aPl = Pl_IterateInit(NULL); aPl != NULL; aPl = Pl_IterateNext()) {

    if( aPl->cmp == MSGC || aPl->cmp == OUTER_TRACKER ) {

       if( (itsSL = SLLookup_bp(aPl->cmp, aPl->l2npl)) == NULL ){
	 sprintf(mess,"Geo_GetBack: Something's seriously wrong\n");
	 grr_error(GRR_SFATAL,mess,1);
	 exit(EXIT_FAILURE);
       }

       if((itsSL->name[0] == 'P' && itsSL->name[1] == 'C') ||
          (itsSL->name[0] == 'T' && itsSL->name[1] == 'C') ||
          (itsSL->name[0] == 'M' && itsSL->name[1] == 'S' && itsSL->name[2] == '1')) {

        switch (which) {

        case GEO_GET_ALL:
          plArray[nPl++] = aPl;
          break;

        case GEO_GET_X:
          if ( aPl->sin > -.00001 && aPl->sin < .00001) {
	    /*printf("found %d\n", aPl->l2npl);*/
	    plArray[nPl++] = aPl;
          }
	  break;

        case GEO_GET_STEREO:
	  if ( aPl->sin < -.00001 || aPl->sin > .00001)
	  plArray[nPl++] = aPl;
	  break;
        }
      }
    }
  }
  
  return plArray;
}

/***
 *
 * Make an index into plTable. Return the length of the index.
 * Entries start at zero. The length is the plane number of the
 * plane with the highest plane number in plTable plus one.
 *
 ***/

PLANE **Geo_MakeIndex(int *length)
{
  PLANE *aPl, **index;
  int i, highest = 0;
  char mess[200];
  
  /* Find out how much memory to allocate */
  
  for(aPl = Pl_IterateInit(NULL); aPl != NULL; aPl = Pl_IterateNext())
    highest = (aPl->l2npl > highest) ? aPl->l2npl : highest;
  
  if(highest == 0) {
    *length = 0;
    return NULL;
  }
  
  if( (index = (PLANE **) malloc( (highest+1) * sizeof(PLANE *))) == NULL){
    sprintf(mess,"GeoMakeIndex: unable to allocate memory for plane index\n");
    grr_error(GRR_SFATAL,mess,1);
  }

  for(i = 0; i <= highest; i++)
    index[i] = NULL;
  
  for(aPl = Pl_IterateInit(NULL); aPl != NULL; aPl = Pl_IterateNext())
    index[aPl->l2npl] = aPl;

  *length =  highest+1;
  return index;
  
}

/***
 *
 * Geo_GetMag1ZLimits
 *
 * Return (through calling sequence) the minimum and maximum
 * z-values of the set MC01, MS01, SI08.
 *
 * If no chambers exist, return zMin = zMax = zMagDef
 *
 ***/

void Geo_GetMag1ZLimits(float *zMin, float *zMax)
{

  const float zMagDef = 215.;
  int foundSomething = 0;

  SUPERLAYER *smc01, *sms01, *ssi08;

  *zMin =  100000.;
  *zMax = -100000.;

  smc01 = SLLookup_bn( nMC01 );
  if(smc01 != NULL) {
    *zMin = smc01->z1;
    *zMax = smc01->z2;
    foundSomething = 1;
  }

  ssi08 = SLLookup_bn( nSI08 );
  if(ssi08 != NULL) {
    *zMin = ( *zMin < ssi08->z1 ) ? *zMin : ssi08->z1;
    *zMax = ( *zMax > ssi08->z2 ) ? *zMax : ssi08->z2;
    foundSomething = 1;
  }

  sms01 = SLLookup_bn( nMS01 );
  if(sms01 != NULL) {
    *zMin = ( *zMin < sms01->z1 ) ? *zMin : sms01->z1;
    *zMax = ( *zMax > sms01->z2 ) ? *zMax : sms01->z2;
    foundSomething = 1;
  }

  if(foundSomething == 0) {
    *zMin = zMagDef;
    *zMax = zMagDef;
  }
}

/***
   *
   * Geo_MagnetOn
   *
   * Return 1 if the magnet is on, otherwise 0. I haven't found a
   * way to figure out if the magnet is on using Arte, so I just
   * look at the l2mode. So...this only tells if the magnet is
   * supposed to be on, not if it actually (virtually?) is.
   *
   ***/

int Geo_MagnetOn(void)
{
  int mode, dum;

  L2Mode_get(&mode, &dum);

  if(mode > L2_STANDARD) {
    printf(">>>>>> WARNING <<<<<<<< Geo_MagnetOn: unknown l2mode\n");
  }

  if(mode == L2_STANDARD || mode == L2_98_MAGNET_ON || L2_98_FULL_CHAMBERS)
    return 1;

  return 0;
}


/***
 *
 * Geo_GetSiliLimits
 *
 * Return (through calling sequence) the minimum and maximum
 * z-values of vertex detector. The maximum is defined as the
 * exit to the furthest downstream silicon superlayer inside
 * the vertex vessel and the minimum as the entrance to furthest
 * upstream silicon superlayer.
 *
 ***/

void Geo_GetSiliLimits(float *zMin, float *zMax)
{

  SUPERLAYER *ssi;
  int isl;
  char nsi[8];

  *zMin =  100000.;
  *zMax = -100000.;

  for(isl = 0; isl < 7; isl++) {
    sprintf(nsi,"SI0%1d", isl+1);

    if( (ssi = SLLookup_bn( nsi )) == NULL)
      continue;

    if(ssi->z1 < *zMin)
      *zMin = ssi->z1;

    if(ssi->z2 > *zMax)
      *zMax = ssi->z2;

  }


  /* printf("L2 - Silicon detector limits: %f, %f\n", *zMin, *zMax); */

}

/***
 *
 * Geo_GetMag1Index
 *
 * Return an array of pointers to planes in MC01, MS01, SI08
 * These are the planes used by the magnet traversal algorithm
 *
 ***/

PLANE **Geo_GetMag1Index(int *nPlanes)
{
  int ipl, pIndex;
  char mess[200];

  SUPERLAYER *smc01, *sms01, *ssi08;

  PLANE **plPointers;

  smc01 = SLLookup_bn( nMC01 );
  ssi08 = SLLookup_bn( nSI08 );
  sms01 = SLLookup_bn( nMS01 );

  *nPlanes = 0;

  if(smc01 != NULL)
    *nPlanes += smc01->npl;

  if(ssi08 != NULL)
    *nPlanes += ssi08->npl;

  if(sms01 != NULL)
    *nPlanes += sms01->npl;

  if( *nPlanes == 0)
    return NULL;

  if ( (plPointers = (PLANE **) malloc( *nPlanes * sizeof(PLANE *))) == NULL) {
    sprintf(mess," Geo_GetMag1Index: no room for pointer array\n");
    grr_error(GRR_SERROR,mess,1);
  }

  pIndex = 0;

  if(smc01 != NULL) {
    for(ipl = smc01->fpl; ipl < smc01->fpl + smc01->npl; ipl++, pIndex++) {
      plPointers[pIndex] = PlLookup(ipl);
    }
  }

  if(ssi08 != NULL) {
    for(ipl = ssi08->fpl; ipl < ssi08->fpl + ssi08->npl; ipl++, pIndex++) {
      plPointers[pIndex] = PlLookup(ipl);
    }
  }

  if(sms01 != NULL) {
    for(ipl = sms01->fpl; ipl < sms01->fpl + sms01->npl; ipl++, pIndex++) {
      plPointers[pIndex] = PlLookup(ipl);
    }
  }

  return plPointers;

}

/***
 *
 * Geo_GetSiliIndex
 *
 * Return an array of pointers to planes in SI01 through SI07
 * (in descending order starting from Si07)
 *
 ***/

PLANE **Geo_GetSiliIndex(int *nPlanes)
{
  int isl, ipl, pIndex;

  char nsi[8];
  char mess[200];

  SUPERLAYER *ssi[7];

  PLANE **plPointers, *tmp;

  *nPlanes = 0;

  /*** Count the number of planes ***/

  for(isl = 0; isl < 7; isl++) {
    sprintf(nsi,"SI0%1d", isl+1);

    if( (ssi[isl] = SLLookup_bn( nsi )) == NULL)
      continue;

    for(ipl = ssi[isl]->fpl; ipl < ssi[isl]->fpl + ssi[isl]->npl; ipl++) {
      if( PlLookup(ipl) != NULL)
	(*nPlanes)++;
    }
  }

  if(*nPlanes == 0) {
    sprintf(mess,"L2 - Geo_GetSiliIndex Warning, no silicon superlayers found\n");
    grr_error(GRR_SERROR,mess,1);
    return NULL;
  }

  /*** allocate, then fill 'er up ***/

  if ( (plPointers = (PLANE **) malloc( *nPlanes * sizeof(PLANE *))) == NULL) 
    sprintf(mess,"L2 - Geo_GetSiliIndex: no room for pointer array\n");
    grr_error(GRR_SERROR,mess,1);

  pIndex = 0;

  for(isl = 6; isl >= 0; isl--) {

    if(ssi[isl] == NULL)
      continue;

    for(ipl = ssi[isl]->fpl; ipl < ssi[isl]->fpl + ssi[isl]->npl; ipl++) {

      if( (tmp = PlLookup(ipl)) != NULL)
	plPointers[pIndex++] = tmp;

    }
  }
  /*  printf("L2 - Geo_GetSiliIndex found %d silicon planes\n", *nPlanes); */
  return plPointers;
}

/***
 *
 * Geo_GetNsectors
 *
 * Return the number of sectors in the named superlayer
 *
 ***/

int Geo_GetNsectors(const char name[4])
{
  SUPERLAYER *sl;

  if( (sl = SLLookup_bn( name )) == NULL)
    return 0;

  if( sl->lay == 0)
    return 0;

  return sl->npl / sl->lay;
}
/***
 *
 * Geo_GetNlayers
 *
 * Return the number of layers in the named superlayer/sector
 *
 * The number of layers may be sector dependent in the SVD for 98 and,
 * in the "new" OTR geometry of Lanyov, there is more than one GEDE
 * entry per sector/layer. Hence the odd algorithm.
 *
 ***/

int Geo_GetNlayers(const char name[4], int sector)
{
  SUPERLAYER *sl;
  PLANE *pl;
  int ipl;
  int nLayers = 0;
  int il, layMax = 10;
  int lay[10] = {0,0,0,0,0,0,0,0,0,0};

  if( (sl = SLLookup_bn( name )) == NULL)
    return 0;

  if( sl->lay == 0)
    return 0;

  for(ipl = sl->fpl; ipl < sl->fpl + sl->npl; ipl++) {
    if( (pl = PlLookup(ipl)) != NULL && pl->sec == sector) {
      if(pl->lay < layMax)
	lay[pl->lay]++;
    }
  }

  for(il = 0; il < layMax; il++)
    if(lay[il] != 0)
      nLayers++;

  return nLayers;

}

/***
 *
 * Geo_GetPlanes
 *
 * return pointers to all layers in sector
 *
 * Note that sectors are numbered as in TDR, STARTING FROM 1
 *
 ***/

PLANE **Geo_GetPlanes(const char name[4], int sector, int *nLayers)
{
  PLANE **plPointers, *temp;
  char mess[200];
  /* SUPERLAYER *sl; old but not quite ready for the trash can 
  int layer; */

  *nLayers = 0;

  for(temp = Pl_IterateInit(name); temp != NULL; temp = Pl_IterateNext()) {
    if(temp->sec == sector)
      (*nLayers)++;
  }

  if(*nLayers == 0)
    return NULL;

  plPointers = (PLANE **) malloc( (*nLayers) * sizeof(PLANE *));

  if(plPointers == NULL) {
    sprintf(mess,"L2 - Geo_GetPlanes: No room for plane index\n");
    grr_error(GRR_SERROR,mess,1);
    return NULL;
  }

  *nLayers = 0;

  for(temp = Pl_IterateInit(name); temp != NULL; temp = Pl_IterateNext()) {
    if(temp->sec == sector)
      plPointers[(*nLayers)++] = temp;
  }

  return plPointers;

  /**** and this is how we used to do it:

  if( (sl = SLLookup_bn( name )) == NULL)
    return NULL;

  if( sl->lay == 0)
    return NULL;

  if(sector < 1 || sector > sl->npl/sl->lay)
    return NULL;

  for(layer = sl->fpl; layer < sl->fpl + sl->npl; layer++) {

    if( (temp = PlLookup(layer)) == NULL)
      continue;

    if(temp->sec == sector)
      (*nLayers)++;
  }

  if( *nLayers == 0)
    return NULL;

  plPointers = (PLANE **) malloc( (*nLayers) * sizeof(PLANE *));

  *nLayers = 0;
  for(layer = sl->fpl; layer < sl->fpl + sl->npl; layer++) {

    if( (temp = PlLookup(layer)) == NULL)
      continue;

    if(temp->sec == sector)
      plPointers[(*nLayers)++] = temp;
  }

  return plPointers;

  ******/

}

/***
 *
 * Geo_GetViews
 *
 * return number of layers and views in sector
 *
 * Note that sectors are numbered as in TDR, STARTING FROM 1
 *
 * 30.03.01 New version of Geo_GetViews. It returns not only
 *          the view of the layer but also the layer id (Teresa)
 *
 ***/

int Geo_GetViews(const char name[4], int sector, int *view, int *layer)
{
  PLANE *temp;
  int i;
  int countlay[MAX_LAYERS] = {0};
  int nLayers = 0;
  char mess[200];

  for(i=0;i<MAX_LAYERS;i++) countlay[i] = 0;

  for(temp = Pl_IterateInit(name); temp != NULL; temp = Pl_IterateNext()) {
    if(temp->sec == sector && countlay[temp->lay]==0) {
      if (nLayers < MAX_LAYERS) { 
        countlay[temp->lay]++;
        layer[nLayers] = temp->lay;
        if(temp->sin < -0.0001) {
          view[nLayers] = -1;
        } else if(temp->sin > 0.0001)  {
          view[nLayers] =  1;
        } else {
          view[nLayers] = 0;
        }
        nLayers++;
      } else {
	sprintf(mess,"Geo_GetView: Number of Layers above limit !!! \n");
	grr_error(GRR_SERROR,mess,1);
        return 0;
      }
    }
  }
  
  return nLayers;
  
}

/*  30.03.01 Old version of Geo_GetViews is commented and sustituted
             by the one above (Teresa)                              */

//int Geo_GetViews(const char name[4], int sector, int *view)
//{
//  PLANE *temp;
//  int i;
//  int countlay[MAX_LAYERS] = {0};
//  int nLayers = 0;
//
//  for(i=0;i<MAX_LAYERS;i++) countlay[i] = 0;
//
//  for(temp = Pl_IterateInit(name); temp != NULL; temp = Pl_IterateNext()) {
//    if(temp->sec == sector && countlay[temp->lay]==0) {
//      if (nLayers < MAX_LAYERS) {
//        countlay[temp->lay]++;
//        if(temp->sin < -0.0001) {
//          view[nLayers] = -1;
//        } else if(temp->sin > 0.0001)  {
//          view[nLayers] =  1;
//        } else {
//          view[nLayers] = 0;
//        }
//        nLayers++;
//      } else {
//        sprintf(mess,"Geo_GetView: Number of Layers above limit !!! \n");
//	  grr_error(GRR_SERROR,mess,1);
//        return 0;
//      }
//    }
//  }
//
//  return nLayers;
// 
//}


/***
 *
 * Geo_Get98Scenario, Geo_Set98Scenario
 *
 * Get and set scenario number for '98 run. By default, the scenario
 * is -1, meaning full hera-b. For the other possibilites, see
 * hydra: ~rmankel/task/meeting-24-Jun.
 *
 * Set should be called by a special version of l2fr_L2simuInit and
 * Get by the interested l2simu initialization routines. This feature
 * is not implemented in the stand-alone version, but can easily be
 * if there is need.
 *
 ***/

int  Geo_Get98Scenario(void) { return scenario98; }
void Geo_Set98Scenario(int s) { scenario98 = s; }

void Geo_FillVersion( int eventNumber, L2GEO_VERSION *v )
{
  memcpy(v, &dummy_version, sizeof(L2GEO_VERSION));
}

int Geo_GetSectorLimits(const char *slName, int sector, float *xmin, float *xmax, float *ymin, float *ymax)
{
  SUPERLAYER *sl = SLLookup_bn(slName);
  PLANE *pl;
  int i;

  if(sl == 0)
    return -1;

  *xmin = 10000.;
  *xmax = -10000.;
  *ymin = 10000.;
  *ymax = -10000.;

  for(i = sl->fpl; i < sl->fpl + sl->npl; i++) {
    if( (pl = PlLookup(i)) == 0)
      continue;

    if( sector != pl->sec)
      continue;

    if(pl->sxmn < *xmin)
      *xmin = pl->sxmn;

    if(pl->sxmx > *xmax)
      *xmax = pl->sxmx;

    if(pl->symn < *ymin)
      *ymin = pl->symn;

    if(pl->symx > *ymax)
      *ymax = pl->symx;
  }
  if(*xmin > *xmax || *ymin > *ymax)
    return -2;

  return 0;
}










