#ifndef _GEO
#define _GEO
#include "cif.h"
#include "dttable.h"
#include "sltable.h"
#include "pltable.h"

#define MAX_SUPER_LAYERS 80
#define MAX_PLANES 5000
#define MAX_LAYERS 20

struct L2GEO_VERSION {       /* Move this to l2geo */
  unsigned int OTR;    /* Identify geometry used                                    */
  unsigned int ITR;    /* I don't know where to get these but that doesn't          */
  unsigned int CAL;    /* mean we don't need them                                   */
  unsigned int SVD;    /* If calibration is handled separately, we'll need cal ids  */
  unsigned int TGT;
  unsigned int MUO;
  unsigned int HPT;
};
typedef struct L2GEO_VERSION L2GEO_VERSION;

typedef enum {GEO_GET_ALL, GEO_GET_X, GEO_GET_STEREO} GEO_WHICH;

typedef enum {GEO_CNA_INIT, GEO_BINARY_INIT, GEO_ASCII_INIT, GEO_PIPE_INIT} GEO_INIT_MODE;

int         Geo_Init(GEO_INIT_MODE mode, char *file1, char *file2,char *file3);
void        Geo_UpdateGeometry(const CIF_formatted_tab *tab);
int         Geo_Register( void (*callback)(void)); /* registration for geometry updates */
int         init_FtabFromCnA(void);
int         init_GeoFromPipe(void);
int         init_GeoFromBinary(char *fName);
int         init_GeoFromAscii(char *fName);

PLANE      *GetGeo(int, PLANE *);       /* return a copy of a plane's geometry record */

void        Geo_PrintPlane(PLANE *);
PLANE     **Geo_GetBack( GEO_WHICH, int *nPlanes);
PLANE     **Geo_GetMag1Index(int *nPlanes);  /* return pointers to all planes in MC01, MS01, SI08 */
PLANE     **Geo_GetSiliIndex(int *nPlanes);  /* return pointers to all planes in the vertex detector */
PLANE     **Geo_GetPlanes(const char name[4], int sector, int *nLayers); /* return pointers to all layers in sector */
int         Geo_GetViews(const char name[4], int sector, int* view, int* layer); /* returns the number of layers and their view in a given sector (PK) 
    29-March-2001 (Teresa) returns also de relative layer number    */

int         Geo_MagnetOn(void);
void        Geo_GetMag1ZLimits(float *zMin, float *zMax); /* extent in z of 1st super-conglomerate */
void        Geo_GetSiliLimits(float *zMin, float *zMax);  /* extent in z of vertex detector */

int         Geo_GetNsectors(const char name[4]);
int         Geo_GetNlayers(const char name[4], int sector);
int         Geo_GetSectorLimits(const char *sl, int sector, float *xmin, float *xmax, float *ymin, float *ymax);

int         Geo_Get98Scenario(void); /* Scenario number for '98 run */
void        Geo_Set98Scenario(int);

PLANE     **Geo_MakeIndex(int *length); /* Make and return an array of pointers indexed by plane number */

void        Geo_FillVersion( int eventNumber, L2GEO_VERSION *v );

#endif
