/*
 * Set up according to the various modess defined in l2mode.h.
 * The validity of the input parameters is not checked.
 *
 * To be called from from l2init_.
 *
 * For the moment, leave the silicon alone...full silicon (i.e.
 * whatever's defined in geant)
 *
 *
 */
#include "pltable.h"
#include "l2geo.h"
#include "sltable.h"
#include "l2mode.h"

static int l2mode;
static int l2siliconConfiguration;
static int nCalls = 0;

/***
   *
   * magOn is in the calling sequence to cross-check that the magnet setting
   * really corresponds to the selected mode. At the moment, I'm trying to
   * figure out how to find out if the magnet is on, so it isn't implemented
   * yet.
   *
   ***/

void L2Mode_set(int mode, int silicon, int magOn)
{
  if(nCalls++ != 0)
    printf("*********Warning L2Mode_set called more than once**********\n");

  l2mode = mode;
  l2siliconConfiguration = silicon;

  switch(mode) {
  case  L2_98_L1SIM:
    printf( "L2 mode: Fedor's ecal pretrigger, magnet off, no chambers, no CARE\n");
    /*if(magOn) printf("L2Mode_set: oh muh god\n");*/
    break;

  case  L2_98_NO_CHAMBERS:
    printf( "L2 mode: no l1simu, CARE in FLTE, magnet off, no chambers\n");
    break;

  case  L2_98_STANDARD:
    printf( "L2 mode: no l1simu, CARE in FLTE, magnet off, MC01 & MC02\n");
    break;

  case  L2_98_FULL_CHAMBERS:
    printf( "L2 mode: no l1simu, CARE in FLTE, magnet ON, all chambers\n");
    break;

  case L2_98_MAGNET_ON:
    printf( "L2mode: CARE in FLTE, magnet on, no chambers\n");
    break;

  case  L2_STANDARD:
    printf("L2 mode: final HeraB detector, standard l1simu, magnet on\n");
    break;

  default:
    printf("L2 mode: %d unrecognized, changed to standard\n", mode);
    printf("L2 mode: final HeraB detector, standard l1simu, magnet on\n");
    l2mode = L2_STANDARD;
    break;
  }

  /* Note that when Scharein's digitization turn-off code is released the following
     isn't strictly necessary in simulation mode. There might be an optimization
     question though, particularly in trigger-mode.
     */

  l2siliconConfiguration = silicon;

  switch(silicon) {

  case L2_FULL_SILICON:              /* The whole detector */
    printf("L2 silicon plane list: all available detector planes\n");
    break;

  case L2_3456_SILICON:             /* Superlayers 3,4,5, all sectors, all layers */
    printf("L2 silicon plane list: Superlayers 3,4,5, all sectors, all layers\n");

    break;

  case L2_98_STANDARD_SILICON:        /* Layers 3,4,5, sectors 3,4, all layers */
    printf("L2 silicon plane list: Superlayers 3,4,5,6 sectors 3,4, all layers\n");
    break;

  case L2_98_REALISTIC_SILICON:
    printf("L2 silicon plane list: according to S. Masciocchi 5.5.98 (views?) \n");
    break;

  default:
    l2siliconConfiguration = silicon = L2_FULL_SILICON;
    printf("L2 silicon configuration: %d unrecognized, changed to standard\n", silicon);
    printf("L2 silicon plane list: all available detector planes\n");
    break;

  }
  /* these aren't ever used by L2-L3 triggers, exclude by default */

  L2_excludeSL( "MOTH" );
  L2_excludeSL( "MAGN" );
  L2_excludeSL( "SZH3" );
  L2_excludeSL( "EPIP" );
  L2_excludeSL( "RPOT" );
  L2_excludeSL( "FELD" );
  L2_excludeSL( "PPIP" );
  L2_excludeSL( "PTL1" );
  L2_excludeSL( "PTL2" );
  L2_excludeSL( "PTL3" );
  L2_excludeSL( "RICH" );
  L2_excludeSL( "TRSL" );
  L2_excludeSL( "TR01" );
  L2_excludeSL( "TR02" );
  L2_excludeSL( "MUON" );
  L2_excludeSL( "MUD1" );
  L2_excludeSL( "MUD2" );
  L2_excludeSL( "MUD3" );
  L2_excludeSL( "MUD4" );
  L2_excludeSL( "MXD1" );
  L2_excludeSL( "MXD2" );
  L2_excludeSL( "MXD3" );
  L2_excludeSL( "MXD4" );
  
  /* and for '98 ecal only studies, get rid of everything except (maybe) MC01/MC02 */

  if( mode == L2_98_L1SIM || mode == L2_98_NO_CHAMBERS || mode == L2_98_STANDARD || mode == L2_98_MAGNET_ON) {

    Geo_Set98Scenario(1);   /* As defined by "98 task force study" */

    L2_excludeSL( "MC03" );
    L2_excludeSL( "MC04" );
    L2_excludeSL( "MC05" );
    L2_excludeSL( "MC06" );
    L2_excludeSL( "MC07" );
    L2_excludeSL( "MC08" );
    L2_excludeSL( "PC01" );
    L2_excludeSL( "PC02" );
    L2_excludeSL( "PC03" );
    L2_excludeSL( "PC04" );
    L2_excludeSL( "TC01" );
    L2_excludeSL( "TC02" );
    L2_excludeSL( "MS01" );
    L2_excludeSL( "MS03" );
    L2_excludeSL( "MS05" );
    L2_excludeSL( "MS06" );
    L2_excludeSL( "MS07" );
    L2_excludeSL( "MS10" );
    L2_excludeSL( "MS11" );
    L2_excludeSL( "MS12" );
    L2_excludeSL( "MS13" );
    L2_excludeSL( "MS14" );
    L2_excludeSL( "MS15" );


    if( mode != L2_98_STANDARD) {
      L2_excludeSL( "MC01" );
      L2_excludeSL( "MC02" );
    }
  }

  if(mode != L2_STANDARD) /* ... needs some work, probably a few more modes */
    /*    L2_excludeSL( "SI08" );*/
  /* For the time being, exclude the ITR from full detector studies (PK) */
  L2_excludeSL( "MS03" );
  L2_excludeSL( "MS05" );
  L2_excludeSL( "MS06" );
  L2_excludeSL( "MS07" );
  /* Up to here the planes were excluded */
  
  /*
  L2_excludeSL( "MS01" );
  L2_excludeSL( "MS10" );
  L2_excludeSL( "MS11" );
  L2_excludeSL( "MS12" );
  L2_excludeSL( "MS13" );
  L2_excludeSL( "MS14" );
  L2_excludeSL( "MS15" );
  */

  /*
  L2_excludePlane("MS10",1,1);
  L2_excludePlane("MS10",2,1);
  L2_excludePlane("MS10",1,2);
  L2_excludePlane("MS10",2,2);
  L2_excludePlane("MS10",1,3);
  L2_excludePlane("MS10",2,3);
  L2_excludePlane("MS10",1,4);
  L2_excludePlane("MS10",2,4);
  L2_excludePlane("MS10",1,5);
  L2_excludePlane("MS10",2,5);
  L2_excludePlane("MS10",1,6);
  L2_excludePlane("MS10",2,6);
  L2_excludePlane("MS10",1,7);
  L2_excludePlane("MS10",2,7);
  L2_excludePlane("MS10",1,8);
  L2_excludePlane("MS10",2,8);
  */
 
  /*
  L2_excludePlane("MS10",3,1);
  L2_excludePlane("MS10",4,1);
  L2_excludePlane("MS10",5,1);
  L2_excludePlane("MS10",6,1);
  L2_excludePlane("MS10",7,1);
  L2_excludePlane("MS10",8,1);
  L2_excludePlane("MS10",3,2);
  L2_excludePlane("MS10",4,2);
  L2_excludePlane("MS10",5,2);
  L2_excludePlane("MS10",6,2);
  L2_excludePlane("MS10",7,2);
  L2_excludePlane("MS10",8,2);
  L2_excludePlane("MS10",3,3);
  L2_excludePlane("MS10",4,3);
  L2_excludePlane("MS10",5,3);
  L2_excludePlane("MS10",6,3);
  L2_excludePlane("MS10",7,3);
  L2_excludePlane("MS10",8,3);
  L2_excludePlane("MS10",3,4);
  L2_excludePlane("MS10",4,4);
  L2_excludePlane("MS10",5,4);
  L2_excludePlane("MS10",6,4);
  L2_excludePlane("MS10",7,4);
  L2_excludePlane("MS10",8,4);
  L2_excludePlane("MS10",3,5);
  L2_excludePlane("MS10",4,5);
  L2_excludePlane("MS10",5,5);
  L2_excludePlane("MS10",6,5);
  L2_excludePlane("MS10",7,5);
  L2_excludePlane("MS10",8,5);
  L2_excludePlane("MS10",3,6);
  L2_excludePlane("MS10",4,6);
  L2_excludePlane("MS10",5,6);
  L2_excludePlane("MS10",6,6);
  L2_excludePlane("MS10",7,6);
  L2_excludePlane("MS10",8,6);
  L2_excludePlane("MS10",3,7);
  L2_excludePlane("MS10",4,7);
  L2_excludePlane("MS10",5,7);
  L2_excludePlane("MS10",6,7);
  L2_excludePlane("MS10",7,7);
  L2_excludePlane("MS10",8,7);
  L2_excludePlane("MS10",3,8);
  L2_excludePlane("MS10",4,8);
  L2_excludePlane("MS10",5,8);
  L2_excludePlane("MS10",6,8);
  L2_excludePlane("MS10",7,8);
  L2_excludePlane("MS10",8,8);
  */

  /*
  L2_excludePlane("PC01",4,1,13);
  L2_excludePlane("PC01",4,2,13);
  L2_excludePlane("PC01",4,3,13);
  L2_excludePlane("PC01",4,4,16);
  L2_excludePlane("PC01",4,5,16);
  L2_excludePlane("PC01",4,6,16);
  L2_excludePlane("PC02",4,1,14);
  L2_excludePlane("PC02",4,2,15);
  L2_excludePlane("PC02",4,3,14);
  L2_excludePlane("PC02",4,4,14);
  L2_excludePlane("PC02",4,5,14);
  L2_excludePlane("PC02",4,6,15);
  L2_excludePlane("PC03",4,1,14);
  L2_excludePlane("PC03",4,2,15);
  L2_excludePlane("PC03",4,3,14);
  L2_excludePlane("PC03",4,4,14);
  L2_excludePlane("PC03",4,5,14);
  L2_excludePlane("PC03",4,6,15);
  L2_excludePlane("PC04",4,1,13);
  L2_excludePlane("PC04",4,2,13);
  L2_excludePlane("PC04",4,3,13);
  L2_excludePlane("PC04",4,4,17);
  L2_excludePlane("PC04",4,5,16);
  L2_excludePlane("PC04",4,6,16);
  */
  /* PK: Here comes a special-exclude for Nov 99 studies */
  /*
  L2_excludeSL( "PC02" );
  L2_excludeSL( "PC03" );
  L2_excludeSL( "TC01" );
  L2_excludeSL( "TC02" );
  printf("Excluded PC02,PC03,TC01,TC02 !\n"); 
  */
  Pl_PrintExcludeTable(); 

  /* Next set up silicon ... */

  switch(silicon) {

  case (L2_FULL_SILICON):
    return;                /* note return */

  case (L2_3456_SILICON):
    L2_excludeSL( "SI01" );
    L2_excludeSL( "SI02" );
    L2_excludeSL( "SI07" );
    break;

  case (L2_98_STANDARD_SILICON):
    L2_excludeSL( "SI01" );
    L2_excludeSL( "SI02" );
    L2_excludeSL( "SI07" );

    /*    L2_excludePlane("SI03", 1, 0);
    L2_excludePlane("SI04", 1, 0);
    L2_excludePlane("SI05", 1, 0);
    L2_excludePlane("SI06", 1, 0);
    L2_excludePlane("SI03", 2, 0);
    L2_excludePlane("SI04", 2, 0);
    L2_excludePlane("SI05", 2, 0);
    L2_excludePlane("SI06", 2, 0); */
    break;

  case (L2_98_REALISTIC_SILICON):

    /* 3: 2-1d, 3-1d; 4: 2-2d, 3-2d; 5: 2-2d, 3-2d 6: 2-2d, 3-1d+1s */
    /* modified to agree with numbering in .geo from W.W. 
       released to study data */

    L2_excludeSL( "SI01" );
    L2_excludeSL( "SI02" );
    L2_excludeSL( "SI07" );
#if 0
    L2_excludePlane("SI03", 1, 0);
    L2_excludePlane("SI03", 2, 3);
    L2_excludePlane("SI03", 2, 4);
    L2_excludePlane("SI03", 3, 3);
    L2_excludePlane("SI03", 3, 4);
    L2_excludePlane("SI03", 4, 0);

    L2_excludePlane("SI04", 1, 0);
    L2_excludePlane("SI04", 4, 0);

    L2_excludePlane("SI05", 1, 0);
    L2_excludePlane("SI05", 4, 0);

    L2_excludePlane("SI06", 1, 0);
    L2_excludePlane("SI06", 3, 3); 
    L2_excludePlane("SI06", 4, 0);
#endif
#ifdef OLD
    /* 3: 3-1d, 4-1d; 4: 3-2d, 4-2d; 5: 3-2d, 4-2d 6: 1-1d, 2-1d+1s, 3-1d+1s, 4-1d */
    L2_excludeSL( "SI01" );
    L2_excludeSL( "SI02" );
    L2_excludeSL( "SI07" );

    L2_excludePlane("SI03", 1, 0);
    L2_excludePlane("SI03", 2, 0);
    L2_excludePlane("SI03", 3, 3);
    L2_excludePlane("SI03", 3, 4);
    L2_excludePlane("SI03", 4, 3);
    L2_excludePlane("SI03", 4, 4);

    L2_excludePlane("SI04", 1, 0);
    L2_excludePlane("SI04", 2, 0);

    L2_excludePlane("SI05", 1, 0);
    L2_excludePlane("SI05", 2, 0);

    L2_excludePlane("SI06", 1, 3);
    L2_excludePlane("SI06", 1, 4);
    L2_excludePlane("SI06", 2, 3);
    L2_excludePlane("SI06", 2, 4);
    L2_excludePlane("SI06", 3, 4);
    L2_excludePlane("SI06", 4, 4);
#endif

    break;
  }

  Pl_PrintExcludeTable();
}

void L2Mode_get(int *mode, int *silicon)
{
  *mode = l2mode;
  *silicon  = l2siliconConfiguration;
}




