#ifndef _L2TRIGTAB
#define _L2TRIGTAB

#define L2T_NAME_LENGTH 20

typedef struct {
  char name[L2T_NAME_LENGTH];
  unsigned char  fcs_type;
  unsigned char  flt_type;
  unsigned short slt_type;
  unsigned int control;
  unsigned int prescale;
  unsigned char n_ecal;    /* number of input rois from ecal without gamma flag set */
  unsigned char n_gamma;   /* number of input rois from ecal with gamma flag set */
  unsigned char n_muon;    /* number of input rois required from muon */
  unsigned char n_hipt;    /* number of input rois required from hipt */
  unsigned char n_ecal_h;  /* number of input rois from ecal without gamma but with high_pt flag set */
  unsigned char n_gamma_h; /* number of input rois from ecal with gamma and hi_pt flag set */
  unsigned char n_muon_h;  /* number of input rois required from muon with hi_pt flag set */
  unsigned char n_hipt_h;  /* number of input rois required from hipt with hi_pt flag set */
  float pt;                /* "normal" pt threshold  */
  float pt_hi;             /* threshold for hi_pt flag  */
  float min_mass;
  float ip_lo;    /* if ip cut, all tracks must have impact parameter > ip_lo  */
  float ip_hi;    /* if ip cut, at least one track must have impact parameter > ip_hi  */
  float dz_vtx;   /* if detach cut, min distance between wire and vertex */
} L2TENTRY;

/* the bit definitions in control: */

#define L2CTL_ENABLE       0x1
#define L2CTL_FLT_REC      0x2   /* if FLT record is to be used for seed definitions */
#define L2CTL_OUTBEND     0x04   /* if the outbender requirement is to be applied -- 2tk triggers */
#define L2CTL_LIKE        0x08   /* if like signs are required -- 2 tk triggers */
#define L2CTL_UNLIKE      0x10   /* if unlike signs are required -- 2 tk triggers */
#define L2CTL_IP          0x20   /* if impact parameter cuts are required */
#define L2CTL_VTX         0x40   /* if a vertex requirement is made */
#define L2CTL_DETACH     0x080   /* if the vertex should be away from the wire */


/************** Symbol table definition ***************/

typedef struct {
  char group[L2T_NAME_LENGTH]; /* maybe a pak name or an algo section for example */
  char name[L2T_NAME_LENGTH];  /* unique name within group to identify parameter */
  int data_type; /* int = 0, float = 1 */
  int i_value;
  float f_value;
} L2SYMBOL;

/***  trigger table routines  ***/

int L2TT_ReadFromFile(char *fName, L2TENTRY **tab, int *nEntries);  /* memory is allocated & given to caller */
int L2TT_ReadFromDB(char *dbName, int key, L2TENTRY **tab, int *nEntries, int major, int minor); /* memory as above */
int L2TT_WriteToFile(char *fName, L2TENTRY *tab, int nEntries);
int L2TT_WriteToDB(char *dbName, int key, L2TENTRY *tab, int nEntries);
int L2TT_UpdateTable(L2TENTRY *newTab, int nEntries); /* assumes ownership of newTab and that it was malloced */

int L2TT_CheckTable(L2TENTRY *tab);
int L2TT_CheckEntry(L2TENTRY *entry);

const L2TENTRY *L2TT_getTrigTable(int *nEntries);

int L2TT_getTrigEntry(char *name, L2TENTRY **settings); /* get the parameters of a given type of trigger */

/*** symbol table routines  ***/

int L2TS_ReadFromFile(char *fName, L2SYMBOL **tab, int *nEntries);  /* memory is allocated & given to caller */
int L2TS_ReadFromDB(char *dbName, int key, L2SYMBOL **tab, int *nEntries, int major, int minor); /* memory as above */
int L2TS_WriteToFile(char *fName, L2SYMBOL *tab, int nEntries);
int L2TS_WriteToDB(char *dbName, int key, L2SYMBOL *tab, int nEntries);
int L2TS_UpdateTable(L2SYMBOL *newTab, int nEntries);

int L2TS_CheckTable(L2SYMBOL *tab);
int L2TS_CheckEntry(L2SYMBOL *entry);

const L2SYMBOL *L2TS_getTable(int *nEntries); /* return full table */
const L2SYMBOL *L2TS_getGroup(char *group, int *nEntries); /* return table for a particular group */
const L2SYMBOL *L2TS_getEntry(char *group, char *name); /* lookup a particular entry */

int L2TS_getIntValue(char *group, char *name); /* get the integer value from a particular entry */

float L2TS_getFloatValue(char *group, char *name); /* get the float value from a particular entry  */

#endif



