#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "grr.h"
#include "sltable.h"
#include "pltable.h"

typedef PLANE Item;

typedef struct melement{
	Item *item;
	struct melement *next;
} Element;

static int numElements = 0;
/*static char *listType = "Plane geometry table";*/

static Element *iteration;
static int itercmp = -1;
static float iterz1, iterz2;

static Element *headElement = NULL;
static Element *lastElement = NULL;

Item *PlLookup(int pl)
{
  Element *anElement = headElement;
  Item *item;
  
  for(anElement = headElement; anElement != NULL; anElement = anElement->next) {
    item = anElement->item;
    if( pl == item->l2npl ) {
      return item;
    }
  }
	
  return NULL;
}

Item *PlLookup_bn(char spl[4],int sec,int lay,int mod)
{
  Element *anElement = headElement;
  Item *item;
  SUPERLAYER *splyr;

  for(anElement = headElement; anElement != NULL; anElement = anElement->next) {
    item = anElement->item;
    splyr= SLLookup_bp(item->cmp,item->l2npl);
    
       if( lay == item->lay && sec == item->sec && mod == item->mod && 
	*((int *)spl) ==  *((int *)splyr->name)   ) {
      return item;
    }
  }
	
  return NULL;
}
#if 0
Item *PlLookup_hwc(char hwc[11])
{
  Element *anElement = headElement;
  Item *item;

  for(anElement = headElement; anElement != NULL; anElement = anElement->next) {
    item = anElement->item;
    
       if((int *)item->hwc == (int *)hwc) {
      return item;
    }
  }
	
  return NULL;
}
#endif


void PlInsert(Item *theItem)
{
	Element *newElement;

	/* Insert only if its superplane is in the superplane table */
	if( SLLookup_bp(theItem->cmp, theItem->l2npl) == NULL)
	  return;
	

	/* Check that it's on on the list of excluded planes */
	if( Pl_isExcluded( theItem))
	  return;

	numElements++;
	
	newElement = malloc( sizeof(Element));
	if(headElement == NULL) {
		headElement = newElement;
		headElement->next = NULL;
		lastElement = headElement;
	} else {
		lastElement->next = newElement;
		lastElement = lastElement->next;
		lastElement->next = NULL;
	}
		lastElement->item = theItem;
}

int PlGetNbEntries()
{
  Element *anElement;
  int nentries = 0;
  for( anElement = headElement; anElement != NULL; anElement = anElement->next) {
    nentries++;
  }
  return nentries;
}

void Pl_destroy(void)
{
  struct melement *elem, *next;

  elem = headElement;

  while( elem != NULL ) {
    free(elem->item);
    next = elem->next;
    free(elem);
    elem = next;
  }

  numElements = 0;
  headElement = NULL;
  lastElement = NULL;
  iteration = NULL;
}

/***
 *
 * Pl_PrintTable
 *
 * Print out the plane table,
 * e.g. for generation of an ascii file
 *
 ***/
void Pl_PrintTable(FILE *fd)
{
	Item *aType;
	Element *anElement;
	
	fprintf(fd,"\\PLANES  %-d\n", numElements);
	fprintf(fd,"l2npl   l2nsl   cmp   lay   sec   mod   shp   \n");
	fprintf(fd,"xmin     xmax     ymin     ymax     zmin     zmax     \n");
	fprintf(fd,"sxmn     sxmx     symn     symx     cos      sin      \n");
	fprintf(fd,"rot0     rot1     rot2     rot3     rot4     ");
	fprintf(fd,"rot5     rot6     rot7     rot8\n");
	fprintf(fd,"acc      eff      dod      pit      ms      \n");
	fprintf(fd,"xwi0     xwi1     xwi2     xwi3\n");
	fprintf(fd,"zwi0     zwi1     zwi2     zwi3\n");
	fprintf(fd,"nwi0     nwi1     nwi2     nwi3\n");
	fprintf(fd,"nsl\n");

	for( anElement = headElement; anElement != NULL; anElement = anElement->next) {
		aType = anElement->item;
		Pl_PrintPlane(fd, aType->l2npl );
	}
}

/***
 *
 * Pl_PrintPlane
 *
 * Print out the geometry of one plane       
 *
 ***/

void Pl_PrintPlane(FILE *fd, int plNumber)
{
  PLANE *pl;

  if( (pl = PlLookup(plNumber)) == NULL){
    fprintf(stderr,"Pl_PrintPlane: can't find plane %-d\n", plNumber);
    exit(1);
  }

  fprintf(fd, "\n");

  fprintf(fd, "%-7d %-7d %-5d %-5d %-5d %-5d %-5d\n",
  pl->l2npl, pl->l2nsl, pl->cmp, pl->lay, pl->sec, pl->mod, pl->shp);

  fprintf(fd, "%-9.4f %-9.4f %-9.4f %-9.4f %-9.4f %-9.4f\n",
  pl->xmin, pl->xmax, pl->ymin, pl->ymax, pl->zmin, pl->zmax);

  fprintf(fd, "%-9.4f %-9.4f %-9.4f %-9.4f %-9.6f %-9.6f\n",
  pl->sxmn, pl->sxmx, pl->symn, pl->symx, pl->cos, pl->sin);

  fprintf(fd, "%-8.4f %-8.4f %-8.4f %-8.4f %-8.4f %-8.4f %-8.4f %-8.4f %-8.4f\n",
  pl->rot[0], pl->rot[1], pl->rot[2], pl->rot[3], pl->rot[4],
  pl->rot[5], pl->rot[6], pl->rot[7], pl->rot[8]);

  fprintf(fd, "%-9.4f %-9.4f %-9.4f %-9.6f %-e\n",
  pl->acc, pl->eff, pl->dod, pl->pit, pl->ms);

  fprintf(fd, "%-9.4f %-9.4f %-9.4f %-9.4f\n",
  pl->xfw[0], pl->xfw[1], pl->xfw[2], pl->xfw[3]);

  fprintf(fd, "%-9.4f %-9.4f %-9.4f %-9.4f\n",
  pl->zfw[0], pl->zfw[1], pl->zfw[2], pl->zfw[3]);

  fprintf(fd, "%-8d %-8d %-8d %-8d\n",
  pl->nwi[0], pl->nwi[1], pl->nwi[2], pl->nwi[3]);

  fprintf(fd, "%-5d \n", pl->nsl);

  if(fd==stdout) fflush(fd); 
}


/***
 *
 * Pl_PrintPl
 *
 * Print out the geometry of one plane       
 *
 ***/

void Pl_PrintPl(FILE *fd, Item *pl)
{

  if(pl == NULL){
    fprintf(stderr,"Pl_PrintPlane: can't find plane\n");
    exit(1);
  }

  fprintf(fd, "\n");

  fprintf(fd, "%-7d %-7d %-5d %-5d %-5d %-5d %-5d\n",
  pl->l2npl, pl->l2nsl, pl->cmp, pl->lay, pl->sec, pl->mod, pl->shp);

  fprintf(fd, "%-9.4f %-9.4f %-9.4f %-9.4f %-9.4f %-9.4f\n",
  pl->xmin, pl->xmax, pl->ymin, pl->ymax, pl->zmin, pl->zmax);

  fprintf(fd, "%-9.4f %-9.4f %-9.4f %-9.4f %-9.6f %-9.6f\n",
  pl->sxmn, pl->sxmx, pl->symn, pl->symx, pl->cos, pl->sin);

  fprintf(fd, "%-8.4f %-8.4f %-8.4f %-8.4f %-8.4f %-8.4f %-8.4f %-8.4f %-8.4f\n",
  pl->rot[0], pl->rot[1], pl->rot[2], pl->rot[3], pl->rot[4],
  pl->rot[5], pl->rot[6], pl->rot[7], pl->rot[8]);

  fprintf(fd, "%-9.4f %-9.4f %-9.4f %-9.6f %-e\n",
  pl->acc, pl->eff, pl->dod, pl->pit, pl->ms);

  fprintf(fd, "%-9.4f %-9.4f %-9.4f %-9.4f\n",
  pl->xfw[0], pl->xfw[1], pl->xfw[2], pl->xfw[3]);

  fprintf(fd, "%-9.4f %-9.4f %-9.4f %-9.4f\n",
  pl->zfw[0], pl->zfw[1], pl->zfw[2], pl->zfw[3]);

  fprintf(fd, "%-8d %-8d %-8d %-8d\n",
  pl->nwi[0], pl->nwi[1], pl->nwi[2], pl->nwi[3]);

  fprintf(fd, "%-5d \n", pl->nsl);

  if(fd==stdout) fflush(fd); 
}

/***
 *
 * Pl_formatCNA
 *
 * Fill the plane section of L2FORMMAP 
 * 
 *
 ***/
void Pl_formatCNA(PLANE * plane)
{
        Element *anElement;
        
        for( anElement = headElement; anElement != NULL; anElement = anElement->next) {
	  *plane = *anElement->item;
	  plane++;
        }
}

/***
 *
 * Read one PLANE structure from an ascii file.
 *
 ***/

void Pl_ReadPlane(FILE *fd, PLANE **Ppl)
{
  int c;
  PLANE *pl;
  char mess[200];
    
  pl = malloc( sizeof(PLANE));
  if(pl == NULL){
     printf("pltable: unable to allocate memory for planes\n");
     exit(EXIT_FAILURE);
  }

  c = fscanf(fd, " %d %d %d %d %d %d %d\n", 
  &pl->l2npl, &pl->l2nsl, &pl->cmp, &pl->lay, &pl->sec, &pl->mod, &pl->shp);
  c = fscanf(fd, "%f %f %f %f %f %f ", 
  &pl->xmin, &pl->xmax, &pl->ymin, &pl->ymax, &pl->zmin, &pl->zmax);
  c = fscanf(fd, "%f %f %f %f %f %f ", 
  &pl->sxmn, &pl->sxmx, &pl->symn, &pl->symx, &pl->cos, &pl->sin);
  c = fscanf(fd, "%f %f %f %f %f %f %f %f %f", 
  &pl->rot[0], &pl->rot[1], &pl->rot[2], &pl->rot[3], &pl->rot[4],
  &pl->rot[5], &pl->rot[6], &pl->rot[7], &pl->rot[8]);
  c = fscanf(fd, "%f %f %f %f %e", 
  &pl->acc, &pl->eff, &pl->dod, &pl->pit, &pl->ms);
  c = fscanf(fd, "%f %f %f %f", 
  &pl->xfw[0], &pl->xfw[1], &pl->xfw[2], &pl->xfw[3]);
  c = fscanf(fd, "%f %f %f %f", 
  &pl->zfw[0], &pl->zfw[1], &pl->zfw[2], &pl->zfw[3]);
  c = fscanf(fd, "%d %d %d %d", 
  &pl->nwi[0], &pl->nwi[1], &pl->nwi[2], &pl->nwi[3]);
  c = fscanf(fd, "%d \n", &pl->nsl);  
  
  if( c == EOF) {
    sprintf(mess,"Pl_readplane: EOF encountered while reading plane records\n"); 
    grr_error(GRR_SFATAL,mess,1);
    exit(EXIT_FAILURE);
  }
  
  *Ppl = pl;
}
/***
 *
 * Pl_PrintPlaneSpy
 *
 * Print out the geometry of one plane for Spy      
 *
 ***/

void Pl_PrintPlaneSpy(FILE *fd, int plNumber)
{
  PLANE *pl;

  if( (pl = PlLookup(plNumber)) == NULL){
    fprintf(stderr,"Pl_PrintPlane: can't find plane %-d\n", plNumber);
    exit(1);
  }

  fprintf(fd, "%4d, %2hd, %2hd, %8.5f, %8.5f, %7.2f, %7.2f, %7.2f, %7.2f, %7.2f,\n",
	  pl->l2npl, pl->shp, pl->cmp, pl->cos, pl->sin, pl->sxmn, pl->sxmx,
	  pl->symn, pl->symx, pl->zfw[0]);

}


/***
 *
 * Pl_IterateInit
 *
 * Set iteration to point to the first element of the list. Return it's iterm
 *
 * if name is not NULL, assume it is a superlayer name and return only planes
 * belonging to that superlayer.
 *
 ***/

Item *Pl_IterateInit(const char *name)
{
  SUPERLAYER *sl;
  Element *anElement;
  Item *item;


  if(name != NULL) {
    sl = SLLookup_bn( name );
    if(sl == NULL) {
      iteration = NULL;
      return NULL;
    }
    itercmp = sl->cmp;
    iterz1 = sl->z1;   /* Checking that z lies in range is not ideal. But I can't
                          think of another way, since the gesl relation in the gede
                          bank doesn't seem to point to the right layer in a 
                          reduced scenario. */
    iterz2 = sl->z2;
  } else {
    itercmp = -1;
  }

  iteration = headElement;

  if(itercmp != -1) {
    for(anElement = iteration; anElement != NULL; anElement = anElement->next) {
      item = anElement->item;
      if( item->cmp == itercmp && item->zmin >= iterz1 && item->zmax <= iterz2 ) {
	iteration = anElement;
	return item;
      }
    }
    return NULL;
  } else
    return (iteration != NULL)? iteration->item : NULL;
}

/***
 *
 * Pl_IterateNext
 *
 * return next element on list, increment the iteration.
 *
 ***/

Item *Pl_IterateNext()
{
  Element *anElement;
  Item *item;

  if(iteration == NULL)
    return NULL;

  iteration = iteration->next;

  if(itercmp != -1) {
    for(anElement = iteration; anElement != NULL; anElement = anElement->next) {
      item = anElement->item;
      if( item->cmp == itercmp && item->zmin >= iterz1 && item->zmax <= iterz2 ) {
	iteration = anElement;
	return item;
      }
    }
    return NULL;
  } else
    return (iteration != NULL)? iteration->item : NULL;
}








