/* Fill data from SLT emulation into DSLT bank */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "etb.h"
#include "l2io_read.h"

static int l2io_fd = -1;
static char* fifo;
#define MAXTRIES 3
#define NAPTIME 5



static char *SLBbuffer=NULL;
static int SLBbuffersize=0;

static void L2IOPipeFedsStart() {
  int fd, tries;
  char* slt_dom;
  char tmpstr[80];

/*   printf("entering L2IOstart\n"); */

  l2io_fd=-1;
  fd=-1;
  slt_dom = getenv("SLT_DOM");

  if (slt_dom==NULL) 
   {
    printf("Couldn't get environment variable SLT_DOM\n");
    return;
   }
  if (slt_dom[0]=='/')  slt_dom = slt_dom + 1;
  if (slt_dom[strlen(slt_dom)-1]=='/')  slt_dom[strlen(slt_dom)-1]='\0';
  sprintf(tmpstr,"/tmp/fifo_%s",slt_dom);
  fifo = tmpstr;

/*   printf("L2IOstart: fifo name = %s\n",fifo); */

  for (tries=1; tries <= MAXTRIES; tries++)
    {
     if ((fd=open(fifo, O_RDONLY|O_NDELAY))!=-1) break;
     sleep(NAPTIME);
    }
  if (fd==-1)
   {
    printf("Couldn't open fifo %s\n",fifo);
    return;
   }
  if (fcntl(fd,F_SETFL,O_RDONLY)==-1)
   {
    printf("Couldn't set permissions on fifo %s\n",fifo);
    return;
   }
  l2io_fd = fd;
}

int L2IOPipeFeds(int max, ETB_fed *feds,int *n, int *bc) {
  char *buff,*newmem;
  int size;
  struct {
    int cmpid;
    int rev;
    int len;
  } head;

  int done=0;

  int rc=ETB_ok;
  *n=0;
  *bc=0;
  if (l2io_fd==-1) L2IOPipeFedsStart();

  /* Read the sltinfo */
  while(!done) {
    while ((size=read(l2io_fd,&head,sizeof(head)))==0)
      usleep(50000);
    if(size!=sizeof(head)) {
      return ETB_corrupted_event;
    }
    ESP_get_int ( (int *) &head, (int *) &head, sizeof(head)/4);
    if (head.len==0) done=1;
    else {
      
      if (*n >=max) {
	rc=ETB_out_of_memory; 
	return rc;
      }

      if ((*bc)+head.len>SLBbuffersize) { /* Is more memory needed? */
	SLBbuffersize+=100000;
	if (SLBbuffersize>1000000) printf("L2IO: Warning high memory usage!\n");
	newmem=realloc(SLBbuffer,SLBbuffersize);
	if (newmem==NULL) {
	  free(SLBbuffer);
	  SLBbuffer=NULL;
	  SLBbuffersize=0;
	  rc=ETB_out_of_memory;
	  return rc;
	}
	SLBbuffer=newmem;
      }
      
      feds[*n].slb = 0;
      feds[*n].hdr.fedid  = head.cmpid<<8;
      feds[*n].hdr.recid  = head.rev+L2IO_FED_RECORD_ID;
      feds[*n].hdr.length = head.len+sizeof(SLB_SUBHDR);
      feds[*n].hdr.bxid   = 256;

      buff=SLBbuffer+(*bc);
      feds[*n].data=buff;
      (*n)++;
      (*bc)+=head.len;
      
      while ((size=read(l2io_fd,buff,head.len))==0)
        usleep(50000);
      if (size!=head.len) 
	rc=ETB_corrupted_event;
    }
  }
  return rc;
}


