/*---------------------------------------------------*
 *                                                   *
 * routines to get MC truth information from         *
 * FEDs in component MCTRUTH                         *
 * to be used after a call to                        *
 * ETB_get_feds( (ETB_event *)event,                 *
 *           COMP_ID_MCTRUTH, max, feds, &N ) :      *
 *                                                   *
 * mtrack* get_track_info(ETB_fed fed)               *
 * mtpoint* get_mimp_info(ETB_fed fed, short num)    *
 * These structures are defined in mctruth.h         *
 * in the directory                                  *
 * /afs/desy.de/group/hera-b/SLT/slt/Rev0504/include *
 *                                                   *
 * Author: Lars Soezueer <soezueer@mail.desy.de>     *
 * Date: 25-05-2000                                  *
 *---------------------------------------------------*/
/*
$Id: get_MCtruth.C,v 1.1 2000/05/26 17:58:13 soezueer Exp $
$Log: get_MCtruth.C,v $
Revision 1.1  2000/05/26 17:58:13  soezueer
adding get_MCtruth.C

*/
#include <string.h>
#include "component.h"
#include "mctruth.h"

/*------------------------------------------------*
 *                                                *
 * get_track_info returns MC track info           *
 * for the track in fed                           *
 *                                                *
 *------------------------------------------------*/


mtrack* get_track_info(ETB_fed fed)
{
 mtrack *rc;
 // printf("get_track_info: start\n");
 //try without byte swap
 //ESP_get_mtrack((mtrack*)(fed.data), rc);
 rc=(mtrack*)(fed.data);
 if (rc==0) printf("get_track_info: rc=0 !!!\n");
 // printf("get_track_info: numpoints=%d\n",rc->numpoints);
 return rc;
}

/*------------------------------------------------*
 *                                                *
 * get_mimp_info returns info on MC impact point  *
 * for the mtrack.numpoints MIMPs on the track    *
 * in fed                                         *
 *                                                *
 *------------------------------------------------*/

mtpoint* get_mimp_info(ETB_fed fed, short num)
{
 mtpoint *rc;
 if ( num>=0 && num<((mtrack*)(fed.data))->numpoints )
  {
    //   mtpoint* p = (mtpoint*) ((mtrack*)fed.data + 1);
   //try without byte swap
   //   ESP_get_mimp( p+num, rc);
    //   rc=p+num;
   char* p = (char*) ((mtrack*)fed.data + 1);
   int len=sizeof(mtpoint);
   //check if it is old mc truth?
   if (((mtrack*)fed.data)->orig.compid==0) {
     len=sizeof(mtvert);
   }
   rc=(mtpoint*) (p+num*len);
   return rc;
  }
 else return 0;
}

void ESP_put_mimp(mtpoint *local, mtpoint *exp)
{
     ESP_put_short((short*)local,(short*)exp,2);
     ESP_put_int((int*)&local->x,(int*)&exp->x,6);
}

void ESP_put_mtrack(mtrack *local, mtrack *exp)
{
     ESP_put_mimp((mtpoint*)local,(mtpoint*)exp);
     ESP_put_int((int*)&local->mass,(int*)&exp->mass,1);
     ESP_put_short(&local->parentid,&exp->parentid,4);
}

void ESP_get_mimp(mtpoint *exp, mtpoint *local)
{
     printf("ESP_get_mimp: start\n");
     ESP_get_short((short*)exp,(short*)local,2);
     printf("ESP_get_mimp: after ESP_get_short\n");
     ESP_get_int((int*)&exp->x,(int*)&local->x,6);
     printf("ESP_get_mimp: done\n");
}

void ESP_get_mtrack(mtrack *exp, mtrack *local)
{
     printf("ESP_get_mtrack: start\n");
     ESP_get_mimp((mtpoint*)exp,(mtpoint*)local);
     printf("ESP_get_mtrack: after ESP_get_mimp\n");
     ESP_get_int((int*)&exp->mass,(int*)&local->mass,1);
     ESP_get_short(&exp->parentid,&local->parentid,4);
}

