/***
   * mctruth.h
   * definitions of
   * - the content of MCTRUTH FEDs in daqif
   * - access functions
   * - byte swapping routines
   * Author: Lars Soezueer <soezueer@mail.desy.de>
   * roughly following a proposal of Reinhard Eckmann
   * Date: 18/07/2000
   *
   ***/

#include "esp.h"
#include "etb.h"

typedef struct mimp {
  short compid;  /* detector component */
  short nsl;     /* superlayer number in this component */ 
  float x,y,z;   /* space point */
  float tx,ty,p; /* track direction and momentum*sign(pz) at this point */
  short hitb;    /* hit pattern, stereo view in high byte */
} mtpoint;

typedef struct startpnt {
  short compid;  /* detector component */
  short nsl;     /* superlayer number in this component */ 
  float x,y,z;   /* space point */
  float tx,ty,p; /* track direction and momentum*sign(pz) at this point */
} mtvert;

typedef struct mtrack {
  mtvert orig;        /* information at production points  */
  float   mass;        /* mass*charge, which allows to identify particle */
  short   parentid;    /* lund particle code of parent e.g. to select
                          tracks from J/Psi's */
  short   parentindex; /* if parent track is stored as mtrack this 
                          tells you where to find it, else -1
                          (e.g. Bremsstrahlung) 
                          => not tested if this works <= */
  short   parentvtx;   /* produced in primary (1) or secondary vertex (0), 
                          e.g. seperate B->J/Psi from direct J/Psi*/
  short   numpoints;   /* number of points on the track */
} mtrack;

/* The MCTRUTH FEDs from daqif contain one mtrack followed by numpoints x mtpoint */

/* access routines in get_MCtruth.C */

mtrack* get_track_info(ETB_fed fed);
mtpoint* get_mimp_info(ETB_fed fed, short num);

/* byte swapping: */

void ESP_put_mimp(mtpoint *local, mtpoint *exp);
void ESP_put_mtrack(mtrack *local, mtrack *exp);
void ESP_get_mimp(mtpoint *exp, mtpoint *local);
void ESP_get_mtrack(mtrack *exp, mtrack *local);
