/* General C interface to retrieve SLT information 

   Author: Brian Petersen
  
   Nov 13 1999 - Created 

   */

#include "etb.h"
#include "l2io_read.h"
#include "sltinfo.h"


static L2EVENT l2evt,*l2event=&l2evt;

static SLTRC L2Stat=SLT_OK;

static void Find_Best_SubTrk(L2TRACK *pTrk, L2SILISUBTRK **pBestSubTrk);
static void Find_Best_Refit_SubTrk(L2TRACK *pTrk, L2REFITSUBTRK **pBestSubTrk);

/* Private SLT call */

L2EVENT *GetL2ptr()
{
  return l2event;
}

/* public */

SLTRC SLTReadSLT(EVENT_HDR *event)
{
  int rc;
  ETB_fed feds[MAX_SLT_FEDS];
  int num;

  rc = ETB_get_feds( event, COMP_ID_SLT, MAX_SLT_FEDS, feds, &num );
  if (rc!=ETB_ok) return SLT_NODSLTBANK;
  return SLTReadEvent(feds,num);
}

SLTRC SLTReadEvent(ETB_fed *feds,int num) {
  if (ReadEvent(feds,l2event,num)) return L2Stat=SLT_OK;
  return L2Stat=SLT_NODSLTBANK;
}

/* SLTRC SLTReadPipe() { */
/*   L2IOPipeEvent(l2event); */
/*   return SLT_OK; */
/* } */

SLTRC  SLTPipeFeds(int max, ETB_fed *feds,int *n, int *bc) {
  L2IOPipeFeds(max,feds,n, bc);
  return SLT_OK;
}


SLTRC SLTStatus(SLT_STATUS *status)
{
  if (L2Stat!=SLT_OK) return L2Stat;

  status->ntracks=l2event->nTracks;
  status->ntriggers=l2event->nTriggers;
  /* status is not saved for some reason ... */
/*   if (l2event->status==L2EVENT_ACCEPT)  */
/*     status->status=SLTEVENT_ACCEPT; */
/*   else */
/*     status->status=SLTEVENT_REJECT; */
  if (l2event->acceptMask!=0&&l2event->acceptMask!=0x80000000)
    status->status=SLTEVENT_ACCEPT; 
  else 
    status->status=SLTEVENT_REJECT; 
  if (l2event->acceptMask&0x80000000)
    status->status|=SLTEVENT_BYPASS;
  
  return SLT_OK;
}

SLTRC SLTTrigInfo(int trignum,SLT_TRIGGER *trig)
{
  int i;
  
  if (L2Stat!=SLT_OK) return L2Stat;
  
  if (trignum<0||trignum>=l2event->nTriggers) return SLT_OUTOFRANGE;

  trig->type=l2event->Triggers[trignum].trigID;
  trig->info=l2event->Triggers[trignum].flag;

  if (l2event->Triggers[trignum].status==L2TRIGGER_ACCEPT)
    trig->status=SLTTRIG_ACCEPT;
  else
    trig->status=SLTTRIG_REJECT;
  
  trig->ntracks=l2event->Triggers[trignum].nTrks;
  if (trig->ntracks>SLTMAX_TRKS_TRIG) return SLT_DATAERROR;

  for(i=0;i<trig->ntracks;i++)
    trig->trackindex[i]=l2event->Triggers[trignum].TrkIndex[i];
  return SLT_OK;
}

SLTRC SLTVertexInfo(int trignum,SLT_VERTEX *vert)
{
  L2VERTEX *l2vert;

  if (L2Stat!=SLT_OK) return L2Stat;
  
  if (trignum<0||trignum>=l2event->nTriggers) return SLT_OUTOFRANGE;

  l2vert=&l2event->Triggers[trignum].vertex;

  vert->chisq =  l2vert->chisq;
  vert->xv = l2vert->xv;
  vert->yv = l2vert->yv;
  vert->zv = l2vert->zv;
  vert->sigxv = l2vert->sigxv;
  vert->sigyv = l2vert->sigyv;
  vert->sigzv = l2vert->sigzv;
  vert->dxy = l2vert->dxy;
  vert->wire = l2vert->wire;
  vert->dz = l2vert->dz;
  vert->impact = l2vert->impact;
  vert->alongwire = l2vert->alongwire;

  return SLT_OK;
}



SLTRC SLTTrackInfo(int tracknum,SLT_TRACK *track)
{
  L2TKINFO info;
  L2TRACKSTAT status;

  if (L2Stat!=SLT_OK) return L2Stat;
  
  if (tracknum<0||tracknum>=l2event->nTracks) return SLT_OUTOFRANGE;

  track->status= SLTTRACK_REJECT_OTHER;
  
  info=l2event->Tracks[tracknum].info;
  status=l2event->Tracks[tracknum].status;

  if (status!=L2TRACK_ACCEPT&&status!=L2TRACK_REJECT&&status!=L2TRACK_WEAK_ACCEPT) return SLT_OK;
  
  if (info==L2TKINFO_SILICON_OK&&(status==L2TRACK_ACCEPT||status==L2TRACK_WEAK_ACCEPT)) track->status=SLTTRACK_ACCEPT;

  if (info==L2TKINFO_SILICON_BAD) track->status=SLTTRACK_REJECT_SILI;
  return SLT_OK;
}

SLTRC SLTSeedTrackInfo(int tracknum,SLT_SEEDTRACK *track)
{
  FLT_TRACK *fltseed;

  if (L2Stat!=SLT_OK) return L2Stat;
  
  if (tracknum<0||tracknum>=l2event->nTracks) return SLT_OUTOFRANGE;

  fltseed=&(l2event->Tracks[tracknum].FLTTrk);
  
  track->trackID = fltseed->trackID;
  track->energy  = fltseed->energy ;
  track->e_err   = fltseed->e_err  ;
  track->pxpz    = fltseed->pxpz   ;
  track->charge  = fltseed->charge ;
  track->x1      = fltseed->x1     ;
  track->x1_err  = fltseed->x1_err ;
  track->y1      = fltseed->y1     ;
  track->y1_err  = fltseed->y1_err ;
  track->z1      = fltseed->z1     ;
  track->x2      = fltseed->x2     ;
  track->x2_err  = fltseed->x2_err ;
  track->y2      = fltseed->y2     ;
  track->y2_err  = fltseed->y2_err ;
  track->z2      = fltseed->z2     ;
  return SLT_OK;
}



SLTRC SLTSiliTrackInfo(int tracknum,SLT_SILITRACK *track)
{
  L2SILISUBTRK *silitrk;

  if (L2Stat!=SLT_OK) return L2Stat;
  
  if (tracknum<0||tracknum>=l2event->nTracks) return SLT_OUTOFRANGE;

  Find_Best_SubTrk(&(l2event->Tracks[tracknum]),&silitrk);

  track->pxpz = silitrk->MOM;
  track->tx   = silitrk->SLO[0];
  track->bx   = silitrk->INT[0];
  track->ty   = silitrk->SLO[1];
  track->by   = silitrk->INT[1];
  track->nhitx= silitrk->NHIT[0];
  track->nhity= silitrk->NHIT[1];
  track->chix = silitrk->CHI[0];
  track->chiy = silitrk->CHI[1];

  return SLT_OK;
}


SLTRC SLTRefitTrackInfo(int tracknum,SLT_REFITTRACK *track)
{
  L2REFITSUBTRK *refittrk;

  if (L2Stat!=SLT_OK) return L2Stat;
  
  if (tracknum<0||tracknum>=l2event->nTracks) return SLT_OUTOFRANGE;

  Find_Best_Refit_SubTrk(&(l2event->Tracks[tracknum]),&refittrk);

  track->tx   = refittrk->slope[0];
  track->bx   = refittrk->intercept[0];
  track->ty   = refittrk->slope[1];
  track->by   = refittrk->intercept[1];
  track->nhits= refittrk->nUsed;
  track->chisq= refittrk->chi2sum;

  return SLT_OK;
}


SLTRC SLTSiliTrackCovInfo(int tracknum,SLT_SILITRACKCOV *track)
{
  L2SILISUBTRK *silitrk;
  float invdet;

  if (L2Stat!=SLT_OK) return L2Stat;
  
  if (tracknum<0||tracknum>=l2event->nTracks) return SLT_OUTOFRANGE;

  Find_Best_SubTrk(&(l2event->Tracks[tracknum]),&silitrk);

  invdet = 1./(silitrk->SZSQ[0]*silitrk->S0[0]-silitrk->SZ[0]*silitrk->SZ[0]);

  track->xcov[0] =  silitrk->S0[0] * invdet;   /* t  */
  track->xcov[1] = -silitrk->SZ[0] * invdet;   /* tx */
  track->xcov[2] =  silitrk->SZSQ[0] * invdet; /* x  */

  invdet = 1./(silitrk->SZSQ[1]*silitrk->S0[1]-silitrk->SZ[1]*silitrk->SZ[1]);
  track->ycov[0] =  silitrk->S0[1] * invdet;   /* t  */
  track->ycov[1] = -silitrk->SZ[1] * invdet;   /* tx */
  track->ycov[2] =  silitrk->SZSQ[1] * invdet; /* x  */

  return SLT_OK;
}

SLTRC SLTRefitTrackCovInfo(int tracknum,SLT_REFITTRACKCOV *track)
{
  L2REFITSUBTRK *reftrk;
  float invdet;

  
  if (L2Stat!=SLT_OK) return L2Stat;
  
  if (tracknum<0||tracknum>=l2event->nTracks) return SLT_OUTOFRANGE;

  Find_Best_Refit_SubTrk(&(l2event->Tracks[tracknum]),&reftrk);

  invdet = 1./(reftrk->swzz[0]*reftrk->sw[0]-reftrk->swz[0]*reftrk->swz[0]);

  track->xcov[0] =  reftrk->sw[0] * invdet;   /* t  */
  track->xcov[1] = -reftrk->swz[0] * invdet;   /* tx */
  track->xcov[2] =  reftrk->swzz[0] * invdet; /* x  */

  invdet = 1./(reftrk->swzz[1]*reftrk->sw[1]-reftrk->swz[1]*reftrk->swz[1]);
  track->ycov[0] =  reftrk->sw[1] * invdet;   /* t  */
  track->ycov[1] = -reftrk->swz[1] * invdet;   /* tx */
  track->ycov[2] =  reftrk->swzz[1] * invdet; /* x  */

  return SLT_OK;
}

#define N_SILICON_SL 7

static int n_spl(int id) {
  int i,n=0;
  for(i=0;i<N_SILICON_SL;i++) {
    if ((id & (1<<i))>0) ++n;
  } 
  return n;
}

static void Find_Best_SubTrk(L2TRACK *pTrk, L2SILISUBTRK **pBestSubTrk) {

  int i,j;
  L2SILITRK *pSiliTrk;
  L2SILISUBTRK *pSiliSubTrk; 
  int maxnspl=0,maxnhits=0;
 
  for(i=0;i<pTrk->SiliTrkSet.nTracks;i++){
    
    pSiliTrk=&pTrk->SiliTrkSet.l2silitrk[i];
    
    for(j=0;j<pSiliTrk->nTracks;j++){ 
      
      pSiliSubTrk=&pSiliTrk->SiliSubTrk[j];
      
      if (n_spl(pSiliSubTrk->NLTR[0]) >= 2 && n_spl(pSiliSubTrk->NLTR[1]) >= 2 &&
          pSiliSubTrk->NHIT[0] >= 2 && pSiliSubTrk->NHIT[1] >= 2) {
        if( (n_spl(pSiliSubTrk->NLTR[0]) + n_spl(pSiliSubTrk->NLTR[1]) >= maxnspl) && 
            (pSiliSubTrk->NHIT[0] + pSiliSubTrk->NHIT[1] >= maxnhits) ) {
          maxnspl = n_spl(pSiliSubTrk->NLTR[0]) + n_spl(pSiliSubTrk->NLTR[1]);
          maxnhits = pSiliSubTrk->NHIT[0] + pSiliSubTrk->NHIT[1];
          *pBestSubTrk = &pSiliTrk->SiliSubTrk[j];
        } 
      }
    }
  }
  
  return;
}


static void Find_Best_Refit_SubTrk(L2TRACK *pTrk, L2REFITSUBTRK **pBestSubTrk) {

  int i,j;
  L2SILITRK *pSiliTrk;
  L2SILISUBTRK *pSiliSubTrk; 
  L2MAGNETTRK *pMagnet;
  int maxnspl=0,maxnhits=0;
 
  for(i=0;i<pTrk->SiliTrkSet.nTracks;i++){
    
    pSiliTrk=&pTrk->SiliTrkSet.l2silitrk[i];
    
    for(j=0;j<pSiliTrk->nTracks;j++){ 
      
      pSiliSubTrk=&pSiliTrk->SiliSubTrk[j];
      
      if (n_spl(pSiliSubTrk->NLTR[0]) >= 2 && n_spl(pSiliSubTrk->NLTR[1]) >= 2 &&
          pSiliSubTrk->NHIT[0] >= 2 && pSiliSubTrk->NHIT[1] >= 2) {
        if( (n_spl(pSiliSubTrk->NLTR[0]) + n_spl(pSiliSubTrk->NLTR[1]) >= maxnspl) && 
            (pSiliSubTrk->NHIT[0] + pSiliSubTrk->NHIT[1] >= maxnhits) ) {
          maxnspl = n_spl(pSiliSubTrk->NLTR[0]) + n_spl(pSiliSubTrk->NLTR[1]);
          maxnhits = pSiliSubTrk->NHIT[0] + pSiliSubTrk->NHIT[1];
	  pMagnet=pSiliTrk->pMagnetTrk;
          *pBestSubTrk = &(pTrk->ReFitTrkSet.l2refittrk[pMagnet->Refpar].RefitSubTrk[pMagnet->RefSubpar]);
	}
      }
    }
  }
  
  return;
}
